package RPM::Source::BundleImport;

use strict;
use warnings;

our $VERSION='0.078';
#use Cwd;
use Carp;
use File::Basename;
use Source::Package::Tarball 0.001;
use RPM::Source::TransformContainer 0.009;
use RPM::Source::Transformation::Component;
use RPM::Source::Transformation::GenericTransformationSet;
use RPM::Source::Transformation::Factory::RaiseRelease;
use RPM::Source::Transformation::Factory::Flags;
use RPM::Source::Transformation::Factory::BundleImport::SubmitSource;
use RPM::Source::Transformation::Factory::BundleImport::CopySource;
use RPM::Source::Transformation::Factory::BundleImport::ResetMacros;

use RPM::Source::Transformation::RPMEditorProvider;

# TODO move to generic set
use RPM::Source::Transformation::Resource::InputBundle;

our @ISA=qw/RPM::Source::Transformation::Component/;
unshift @RPM::Source::TransformContainer::SUPPORT_CLASSES, __PACKAGE__;

$RPM::Source::Transformation::PkgWriter::outputdir='OUT';
$RPM::Source::Transformation::Factory::RaiseRelease::nextrel='incr';
$RPM::Source::Transformation::Factory::RaiseRelease::defaultchangelog='- updated by package builder';

my $new_package_changelog='- initial import by package builder';
my $regenerated_package_changelog='- regenerated from template by package builder';

our $package_class='Source::Package::Tarball';
# TODO: replace by Package::Tarball get_raw_version_comparator
our $version_comparator;

our $verbose=0;
our $builder_name='';
our $opt_template;
our $opt_die_on_depth=0;
our $log_rejected_names=1;
# to disable dieing on guess_name, set to false
our $dielog_name_guess='reject.GV.guess_name.txt';


my $opt_regenerate;
# TODO:
my ($opt_specname);
our $opt_specprefix;

our @LONGOPT=(
    'regenerate' => \$opt_regenerate,
    );
sub print_longopt {
    print "RPM::Source::BundleImport specific options:\n";
    print "	--regenerate -- regenerate spec file from template.\n";
}

sub register_messages {
    my ($self,$extopt)=@_;
    RPM::Source::Transformation::Component::add_opt($extopt, {
	-template => $opt_template ? $opt_template:
	 ($builder_name ? "templates/${builder_name}-package.spec" : 'templates/package.spec'),
	-specname => $opt_specname,
	-specprefix => $opt_specprefix,
	-opt_regenerate => $opt_regenerate,
	});
}

sub get_spec_parent {
    my (undef,$pkg,$converter_opt,$parentref)=@_;
    my $file=$pkg->file();
    print STDERR "DEBUG: get_spec_parent: processing $file\n" if $verbose > 2;

    $converter_opt->{-opt_bundle_input_file}=$file;
    $converter_opt->{-opt_bundle_url}=$pkg->url;

    my ($bundle_name,$bundle_version)=$package_class->guess_package_name_version($file);
    if (defined $bundle_name) {
	$converter_opt->{-opt_bundle_name}=$bundle_name;
	$converter_opt->{-opt_bundle_version}=$bundle_version;
	print STDERR "guess_package_name_version: bundle_name=$bundle_name bundle_version=$bundle_version\n" if $verbose;
    } else {
	&_log_reject($dielog_name_guess,$file) if $dielog_name_guess;
	die "Oops: guess_package_name_version: strange file $file";
    }

    my $specname=$converter_opt->{-specname};
    my $specprefix=$converter_opt->{-specprefix};
    $specprefix||=$builder_name.'-' if $builder_name;
    $specprefix||='';
    my @SPECARGS=(
	SPECFILE=>$converter_opt->{-template},
	-create_sourcedir=>'tmp',
	-specname=> $specname ? $specname : $specprefix.$converter_opt->{-opt_bundle_name}.'.spec',
	);

    if ($parentref and not $converter_opt->{-opt_regenerate}) {
	$converter_opt->{-no_default_hook}=1;
	@SPECARGS=@$parentref;
    }

    my $parent=$RPM::Source::TransformContainer::RPMEditorProvider->rpmeditoropts2parent($parentref);
    my $spec=$RPM::Source::TransformContainer::RPMEditorProvider->rpmeditoropts2spec(\@SPECARGS);

    ## --could be transformation, if we not kill parent in the end ---
    if ($parent) {
	if ($converter_opt->{-opt_regenerate}) {
	    $converter_opt->{-default_changelog} = $regenerated_package_changelog;
	}
    } else {
	$converter_opt->{-default_changelog} = $new_package_changelog;
    }
    ## --can be transformation -------------------
    if ($parent and $converter_opt->{-opt_regenerate}) {
	my $mainsec=$spec->main_section;
	my $parentmainsec=$parent->main_section;
	foreach my $tag (qw/Serial Epoch Version Release/) {
	    my $val=$parentmainsec->get_tag($tag);
	    $mainsec->set_tag($tag eq 'Serial' ? 'Epoch' : $tag, $val) if defined $val;
	}
    }
    ## --------------------------------------------------
    # existing transformations can change behaviour
    #if ($parent and not $converter_opt->{-opt_regenerate}) {
    #	$parent=undef;
    #}

    return ($spec,$parent);
}

# used in PerlBuilder, MetadataSpecUpdater
sub _log_reject {
    my ($logfile,$name) =@_;
    return unless $log_rejected_names; #and -d dirname($logfile);
    open my $fn, '>>', $logfile || die "$!";
    print $fn $name,"\n";
    close($fn);
}

1;
