package RPM::Source::Transformation::Factory::BundleImport::DependencyAnalyzerCollector;

use strict;
use warnings;
use Carp;

use RPM::Source::Dependency::Analyzer::Collector;
use RPM::Source::Transformation::Resource::InputBundle;
use RPM::Source::Transformation::Resource::DependencyAnalyzer::Storage;
use RPM::Source::Transformation::Factory::DependencyAnalyzer::CollectorCommon;
use RPM::Source::Transformation::Factory::DependencyAnalyzer::CollectorFlags;

use RPM::Source::Transformation::Factory;

our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $verbose=1;
our $default_hint;

sub cli_container_ref {
    return ['RPM::Source::Dependency::Analyzer::Collector'];
}

my $collector;

sub transformations {
    return RPM::Source::Transformation::OptHandlerWrapper->new(
	-priority => PRIORITY_BUNDLE_COLLECT,
	-name => 'sourcedep_collector',
	-opt_handler_sub=>\&opt_handler)
	unless $RPM::Source::Dependency::Analyzer::Collector::opt_disable;
}

sub opt_handler {
    my (undef,undef,undef,$opt,$resourceobj)=@_;
    unless ($collector) {
	$collector=RPM::Source::Dependency::Analyzer::Collector->new(
	    -verbose => $verbose,
	    -hint => $default_hint,
	    );
	print STDERR "DEBUG: BundleImport::DependencyAnalyzerCollector: created collector(-hint=$default_hint)\n" if $verbose>2;
    }
    my $extracted_bundle=$resourceobj->bundle_import_extracted_bundle();
    my $storage=$resourceobj->dependency_analyzer_storage;
    $collector->collect_source_dependencies($storage,$extracted_bundle);
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

1;
