package RPM::Source::Transformation::Factory::BundleImport::FillExecutables;

use strict;
use warnings;
#use Carp;

use RPM::Source::Transformation::Resource::FlagLoader;
use RPM::Source::Transformation::Factory::BundleImport::SetupFlagDB;
use RPM::Source::Transformation::Factory::BundleImport::RegisterPkgMacros;
use RPM::Source::Transformation::Factory;
our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $verbose=0;

&RPM::Source::Transformation::Resource::FlagLoader::register_flagdb('bundleimport/scripts/exclude.txt',FLAG_RESOURCE_LIST_AS_FLAG);
&RPM::Source::Transformation::Resource::FlagLoader::register_flagdb('bundleimport/scripts/no-subpackage.txt',FLAG_RESOURCE_LIST_AS_FLAG);

our $opt_exclude_scripts;
our $opt_executables_subpackage_group='Development/Other';
our $opt_create_executables_subpackage=1;

our @LONGOPT=(
    'exclude-scripts!' => \$opt_exclude_scripts,
    'create-script-section!' => \$opt_create_executables_subpackage,
    );

sub print_longopt {
    print "Fill Executables specific options:\n";
    print "	--exclude-scripts - do not package bundle executables\n";
    print "	--no-exclude-scripts (default)\n";
    print "	--create-script-section\n";
    print "	--no-create-script-section\n";
}

sub register_messages {
    my ($self,$extopt)=@_;
    RPM::Source::Transformation::Component::add_opt($extopt, {
	-opt_executables_subpackage_group => $opt_executables_subpackage_group,
	-opt_executables_subpackage_create => $opt_create_executables_subpackage,
	# undef = unknown, false, true
	-opt_bundle_has_executables => undef,
	-opt_bundle_has_man1 => undef,
	# explicit filenamelists; for future use
	-opt_bundle_executables => undef,
	-opt_bundle_man1 => undef,
	});
}

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_BUNDLE_FILL,
	    -name => 'tarball_update_fill_scripts',
	    -opt_handler_sub=>\&opt_handler_tarball_update_fill_scripts),
	;
}

sub opt_handler_tarball_update_fill_scripts {
    my (undef,$spec,undef,$converter_opt,$resourceobj)=@_;

    #my $installsec=$spec->get_section('install');

    my $has_scripts=$converter_opt->{-opt_bundle_has_executables};
    my $script_has_man1=$converter_opt->{-opt_bundle_has_man1};
    warn 'DEBUG: FillExecutables: has_scripts=',$has_scripts,' has_man1=',$script_has_man1,"\n" if $verbose>4;
    if ($has_scripts) {
	my $filessec=$spec->get_section('files','');
	my $reset_speclist=0;
	if ($opt_exclude_scripts or
	    ($resourceobj->get_flagdb('bundleimport/scripts/exclude.txt') and not defined($opt_exclude_scripts))
	    ) {
	    # drop scripts if no regen
	    foreach my $stype(qw/package description files/) {
		my $sec=$spec->get_section($stype,'scripts');
		$sec->delete() if $sec;
	    }
	    $filessec->exclude_body(qr'^(?:\%exclude\s+)?\%_bindir/');
	    $filessec->exclude_body(qr'^(?:\%exclude\s+)?\%_man1dir/');
	    $filessec->push_body('%exclude %_bindir/*'."\n");
	    # TODO: merge man1 code
	} else {
	    my $descrsection=$spec->get_section('description');
	    my $scriptfilessec=$filessec;
	    my $scriptssec=$spec->get_section('package','scripts');
	    if ($scriptssec) {
		$scriptfilessec=$spec->get_section('files','scripts');
	    } elsif (not $scriptssec and not $resourceobj->get_flagdb('bundleimport/scripts/no-subpackage.txt')
		     and $converter_opt->{-opt_executables_subpackage_create}
		) {
		my $mname=$converter_opt->{-macros_pkg_name};
		$mname//='name';
		$descrsection->push_body('
%package scripts
Summary: %'.$mname.' scripts
Group: '.$converter_opt->{-opt_executables_subpackage_group}.'
Requires: %name = %EVR

%description scripts
scripts for %'.$mname.'
',);
		$filessec->push_body("\n",'%files scripts'."\n");
		$reset_speclist=1;
	    }
	    my $scriptfilessec_has_man1=0;
	    if ($scriptfilessec) {
		$scriptfilessec->visit_body(sub {$scriptfilessec_has_man1=1 if /^\%_man1dir/});
		$scriptfilessec->exclude_body(qr'^(?:\%exclude\s+)?\%_bindir/');
		$scriptfilessec->push_body('%_bindir/*'."\n");
	    }
	    if (defined($script_has_man1)) {
		if ($scriptfilessec_has_man1 and not $script_has_man1) {
		    $scriptfilessec->exclude_body(qr'^(?:\%exclude\s+)?\%_man1dir/');
		} elsif (not $scriptfilessec_has_man1 and $script_has_man1) {
		    $scriptfilessec->push_body('%_man1dir/*'."\n") if $script_has_man1 and not $scriptfilessec_has_man1;
		}
	    }
	}
	# added %files scripts
	$spec->_reset_speclist() if $reset_speclist;
    }
    
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

1;
