package RPM::Source::Transformation::Factory::BundleImport::MetadataSpecDocUpdater;

use strict;
use warnings;
use Carp;

use File::Basename;
use RPM::Source::Tools::ShellGlob;
use RPM::Source::BundleImport::Metadata;
use RPM::Source::BundleImport::Helpers;
use RPM::Source::Transformation::Resource::InputBundle;
use RPM::Source::Transformation::Factory::BundleImport::MetadataDocCollector;
use RPM::Source::Transformation::Factory 0.006;

our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $verbose=1;

our $opt_metadata_update_doc=1;
# bundle mode; just override %doc
# disable for cautious update
our $opt_metadata_lazy_update_doc=1;

sub transformations {
    my @TRANSFORMATIONS;
    push @TRANSFORMATIONS,
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_SRPM_POST_BEGIN,
	    -name => 'metadata update %doc',
	    -opt_handler_sub=>\&opt_handler_spec_updater),
	if $opt_metadata_update_doc;

    return @TRANSFORMATIONS;
}

sub opt_handler_spec_updater {
    my (undef,$spec,undef,$converter_opt,$resourceobj)=@_;
    my $metacollector=$resourceobj->metadata_collector;
    my $docref=$metacollector->getfirst('docfiles:list');
    if ($opt_metadata_lazy_update_doc) {
	my $filessec=$spec->get_section('files','');
	&__filessec_lazy_update_doc($filessec,$docref);
    } else {
	my $filelist=$resourceobj->bundle_import_rebased_filelist();
	my $depth1dir=$resourceobj->bundle_import_rebased_depth1dir();
	my %filemap=map{$_=>1} @$filelist;
	foreach my $sec ($spec->get_sections()) {
	    next if $sec->get_type ne 'files';
	    my $secname=$sec->get_package_name;
	    my $need_doc;
	    $need_doc=1 if $sec->get_raw_package eq '';
	    # main files or two main pythons
	    $need_doc=1 if $secname=~/^python\d?-module/ and $secname!~/-(?:bench|data|devel|devel-doc|doc|docs|examples|pickles|resources|test|test-examples|testing|tests)$/;
	    print STDERR "DEBUG: SpecDocUpdater: --- updating $secname (",$need_doc?'main':'secondary',") ---\n" if $verbose>1;
	    &__filessec_smart_update_doc($sec,$filelist,$depth1dir,\%filemap,$secname,$need_doc,$need_doc ? $docref: undef);
	}
    }

    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub __check_keep_doc {
    my ($filessec)=@_;
    return $filessec->match_body(qr'^##\s*roboflag:\s*noautoupdate\s+doc\b');
}

sub __filessec_lazy_update_doc {
    my ($filessec,$docref)=@_;
    return unless $docref;
    return if &__check_keep_doc($filessec);
    $filessec->exclude_body(qr'^\%doc\s+');
    $filessec->unshift_body('%doc '.join(' ',@$docref)."\n") if @$docref;
}

sub __filessec_smart_update_doc {
    my ($filessec,$filelist,$depth1dir,$filemap,$secname,$is_need_doc,$docref)=@_;
    return if &__check_keep_doc($filessec);
    my $doc_section_found=0;
    my %doc_to_add;
    %doc_to_add=map{$_=>1} @$docref if $docref;

    $filessec->map_body (sub {
	if (/^(\s*%doc\s+)(.*)$/) {
	    my $docprefix=$1;
	    my @doc=grep{defined $_ and $_!~/^\s*$/} split (/\s+/,$2);
	    $doc_section_found=1;
	    my @filtered_doc;
	    foreach my $doc (@doc) {
		if ($doc=~/[\*\{\}\?]/) {
		    my $globpat=&RPM::Source::Tools::ShellGlob::spec_glob2pat($doc);
		    my @match=grep {/^$globpat$/} @$filelist;
		    if (scalar @match) {
			print STDERR "DEBUG: $secname: $doc matches ",join(',',@match),"\n" if $verbose>2;
			push @filtered_doc, $doc;
			map {delete $doc_to_add{$_}} @match;
			map {delete $doc_to_add{dirname($_)}} @match;
		    } elsif ($doc=~/\//) { #/
			push @filtered_doc, $doc;
			delete $doc_to_add{dirname($doc)};
		    } else {
			print STDERR "info: $secname: removed $doc from %doc - matches nothing\n" if $verbose;
		    }
		} elsif ($filemap->{$doc}) {
		    if ($_SourceBundleImport::helper::KNOWN_CAN_BE_IGNORED{$doc}) {
			print STDERR "info: $secname: removed $doc from %doc: known bane!\n" if $verbose;
		    } else {
			push @filtered_doc, $doc;
			if ($doc_to_add{$doc}) {
			    delete $doc_to_add{$doc};
			} elsif ($depth1dir->{$doc}) {
			    # ok
			} elsif ($_SourceBundleImport::helper::KNOWN_DOCS{$doc}) {
			    # known, but not in doc_to_add - may be watchscripts, not BundleImport.
			} else {
			    print STDERR "info: $secname: %doc add $doc to doc detection!\n" if $is_need_doc and $verbose;
			}
		    }
		} elsif ($depth1dir->{$doc}) {
		    push @filtered_doc, $doc;
		    delete $doc_to_add{$doc} if $doc_to_add{$doc};
		} elsif ($doc=~/\//) { #/
		    push @filtered_doc, $doc;
		} else {
		    print STDERR "info: $secname: removed $doc from %doc - not found.\n" if $verbose;
		}
	    }
	    if (scalar @filtered_doc) {
		$_=$docprefix.join(' ',@filtered_doc)."\n";
	    } else {
		$_='';
	    }
	}
			 });
    if ($docref) {
	if ($doc_section_found) {
	    # assume it was desided not to add earlier.
	    map {delete $doc_to_add{$_}} keys(%_SourceBundleImport::helper::KNOWN_CAN_BE_IGNORED);
	}
	my $doc_to_add=join(' ',sort keys(%doc_to_add));
	if ($doc_to_add) {
	    print STDERR "info: $secname: added to %doc: $doc_to_add\n" if $verbose;
	    my $bodyref=$filessec->get_bodyref();
	    if ($bodyref->[1]=~/^\s*\%doc\s/) {
		chomp $bodyref->[1];
		$bodyref->[1].=' '.$doc_to_add."\n";
	    } else {
		$filessec->unshift_body('%doc '.join(' ',@$docref)."\n");
	    }
	}
    }
}

1;

__END__
