package RPM::Source::Transformation::Factory::BundleImport::PerlGeneratorUpgrade;

use strict;
use warnings;
use Carp;

use RPM::Source::Transformation::Factory;
our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

# $verbose=0;

sub transformations {
    return 
	RPM::Source::Transformation::OptHandlerWrapper->new(
	-priority => PRIORITY_PRE_BUNDLE,
	-name => 'upgrade old PerlBuilder packages',
	-opt_handler_sub=>\&opt_handler_perl_generator_upgrade);
}

sub opt_handler_perl_generator_upgrade {
    my (undef,$spec,undef,$converter_opt,$resourceobj)=@_;
    if ($spec->{GEARDIR}) {
	# WARNING: code is SRPM-only, can't be used under gear
	# (where patches might exist only potentially, as a .gear/rule)
	return RPM::Source::TransformContainer::TRANSFORMATION_OK;
    }
    my $macros=$spec->macros;
    my $mainsec=$spec->main_section;
    my $prepsec=$spec->get_section('prep');
    #my $module_name=$macros->get_raw('module_name');
    my $module_version=$macros->get_raw('module_version');
    if ($module_version) {
	$mainsec->map_body(sub {
	    $_='' if /^\%define\s+module_version\s/;
	    s,^Version:(\s+)\%\{?module_version\}?\b,Version:$1$module_version,;
			   });
	foreach my $sec ($mainsec,$prepsec) {
	    $sec->map_body(sub {
		s/\%\{?module_version\}?\b/%{version}/g;
			   });
	}
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

1;
