package RPM::Source::Transformation::Resource::InputBundle;

use strict;
use warnings;
use Carp;
#use Data::Dumper;

use RPM::Source::BundleImport::BundleCache;
use RPM::Source::Transformation::Resource;
our @ISA=qw/RPM::Source::Transformation::Resource/;

our $verbose=0;

push @RPM::Source::TransformContainer::RESOURCE_CLASSES, __PACKAGE__;

# source tarball or unpacked file or dir.
my $input_bundle_cache;
# external metadata (optional)
# examples are ruby gems' metadata.gz and deb's control.tar.xz
my $metadata_bundle_cache;

sub prepare_resource {
    my (undef,undef,undef,$opt,$resourceobj)=@_;
}

sub destroy_resource {
    my (undef,undef,undef,$opt,$resourceobj)=@_;
    $input_bundle_cache=undef;
    $metadata_bundle_cache=undef;
}

# new interface (__TODO__)

# compat interface

sub _submit_input_bundle {
    my (undef,$bundle,$name)=@_;
    $input_bundle_cache=RPM::Source::BundleImport::BundleCache->create_from_input_bundle($bundle,$name);
    print STDERR "DEBUG: Resource::InputBundle: input prepared.\n" if $verbose>1;
}

sub _submit_metadata_bundle {
    my (undef,$bundle,$name)=@_;
    $metadata_bundle_cache=RPM::Source::BundleImport::BundleCache->create_from_input_bundle($bundle,$name);
    print STDERR "DEBUG: Resource::InputBundle: metadata prepared.\n" if $verbose>1;
}

sub _submit_input_file {
    my (undef,$file)=@_;
    print STDERR "DEBUG: _submit_input_file: got $file\n" if $verbose > 2;
    $input_bundle_cache=RPM::Source::BundleImport::BundleCache->create_from_input_file($file);
    print STDERR "DEBUG: Resource::InputBundle: prepared.\n" if $verbose>1;
}

sub _submit_metadata_file {
    my (undef,$file)=@_;
    print STDERR "DEBUG: _submit_metadata_file: got $file\n" if $verbose > 2;
    $metadata_bundle_cache=RPM::Source::BundleImport::BundleCache->create_from_input_file($file);
    print STDERR "DEBUG: Resource::InputBundle: metadata prepared.\n" if $verbose>1;
}

package Source::Shared::ResourceContainer;

# metadata interface

sub bundle_import_get_metadata_cache {
    my $self=shift;
    Carp::croak "Oops! Resource::InputBundle: metadata not initialized." if not $metadata_bundle_cache;
    return $metadata_bundle_cache;
}

# new interface (__TODO__)

sub __TODO__bundle_import_get_subpackages_with_bundles {
    return ''; # main
}

sub __TODO__bundle_import_get_bundle_cache_for_subpackage {
    Carp::croak "Oops! Resource::InputBundle: not initialized." if not $input_bundle_cache;
    return $input_bundle_cache;
}

sub __TODO__bundle_import_get_metadata_cache_for_subpackage {
    Carp::croak "Oops! Resource::InputBundle: metadata not initialized." if not $metadata_bundle_cache;
    return $metadata_bundle_cache;
}

# compat interface

sub bundle_import_input_bundle {
    my $self=shift;
    Carp::croak "Oops! Resource::InputBundle: not initialized." if not $input_bundle_cache;
    return $input_bundle_cache->bc_get_input_bundle(@_);
}

# sub bundle_import_input_name {
#    my $self=shift;
#    Carp::croak "Oops! Resource::InputBundle: not initialized." if not $input_bundle_cache;
#    return $input_bundle_cache->bc_get_input_name(@_);
#}

sub bundle_import_extracted_bundle {
    my $self=shift;
    Carp::croak "Oops! Resource::InputBundle: not initialized." if not $input_bundle_cache;
    return $input_bundle_cache->bc_get_extracted_bundle(@_);
}
    
sub bundle_import_extracted_tmpdir {
    my $self=shift;
    Carp::croak "Oops! Resource::InputBundle: not initialized." if not $input_bundle_cache;
    return $input_bundle_cache->bc_get_extracted_tmpdir(@_);
}

sub bundle_import_unpack_path {
    my $self=shift;
    Carp::croak "Oops! Resource::InputBundle: not initialized." if not $input_bundle_cache;
    return $input_bundle_cache->bc_get_unpack_path(@_);
}

sub bundle_import_rebased_filelist {
    my $self=shift;
    Carp::croak "Oops! Resource::InputBundle: not initialized." if not $input_bundle_cache;
    return $input_bundle_cache->bc_get_rebased_filelist(@_);
}

sub bundle_import_rebased_depth1dir {
    my $self=shift;
    Carp::croak "Oops! Resource::InputBundle: not initialized." if not $input_bundle_cache;
    return $input_bundle_cache->bc_get_rebased_depth1dir(@_);
}

1;
